import React from "react";
import { loadFeature, defineFeature } from "jest-cucumber";
import { render, screen /* fireEvent, act*/ } from "@testing-library/react";
import App from "../App";
import { server } from "../mocks/server";
import { rest } from "msw";
import { createMemoryHistory } from "history";
// import { Router } from 'react-router-dom'

// mock BrowserRouter (this should probably live somewhere else)
const reactRouterDom = require("react-router-dom");
reactRouterDom.BrowserRouter = ({ children }) => <div>{children}</div>;
const Router = reactRouterDom.Router;

// TODO: Some of these tests don't do anything yet. They should probably fail insted of passing.
// But they do demonstrate how you can use the features in tests.

const feature = loadFeature("features/auth/login.feature");

defineFeature(feature, (test) => {
  test("Opening application when not authenticated", async ({
    given,
    when,
    then
  }) => {
    console.debug(server);
    given("the user is not authenticated", () => {
      // let's set a handler which returns 401 "Unauthorized" when fetching userInfo
      // this is one of the first pieces of info the app tries to fetch
      server.use(
        rest.get("*/api/v1/git_helper/user_info", (req, res, ctx) => {
          return res(ctx.status(401));
        })
      );
    });

    when("the user opens the application", async () => {
      const history = createMemoryHistory();
      history.push("/iocs");
      render(
        <Router history={history}>
          <App />
        </Router>
      );
    });

    then("login button should appear", async () => {
      // check that the login Button is shown
      expect(await screen.findByText(/login/i)).toBeInTheDocument();
    });
  });

  // test('Opening application root or login when already authenticated', ({ given, when, then }) => {
  //   given('the user is already authenticated', () => {
  //     // default mock api setup should return 200 and example data for userInfo
  //   });

  //   when('the user opens the application to the root or login page', () => {
  //     const history = createMemoryHistory()
  //     history.push('/')
  //     render(
  //       <Router history={history}>
  //         <App />
  //       </Router>
  //     )
  //   });

  //   then('the user should redirected to the IOCs list', async () => {
  //     expect(await screen.findByText(/new ioc/i)).toBeInTheDocument()
  //   });
  // });

  // test('Opening application path when already authenticated', ({ given, when, then }) => {
  //   given('the user is already authenticated', () => {
  //     // default api mock should be good enough to make the app think it is logged in.
  //   });

  //   when('the user opens the application with a valid path in the URL', () => {
  //     const history = createMemoryHistory()
  //     history.push('/deployments')
  //     render(
  //       <Router history={history}>
  //         <App />
  //       </Router>
  //     )
  //   });

  //   then('then the content at that path should be displayed without prompting for login', async () => {
  //     expect(await screen.findByText(/my deployments/i)).toBeInTheDocument()
  //   });
  // });

  // test('Entering an incorrect username or password into Login form', ({ given, when, then }) => {
  //   given('the user is on the login page', () => {
  //     const history = createMemoryHistory()
  //     history.push('/login')
  //     render(
  //       <Router history={history}>
  //         <App />
  //       </Router>
  //     )

  //     server.use(
  //       rest.post("*/login", (req, res, ctx) => {
  //           return res(ctx.status(500), ctx.json({
  //             description: 'Bad username/password'
  //           })
  //         );
  //       })
  //     );
  //   });

  //   when('the user enters a username or password which are not valid', async () => {
  //     await screen.findByText(/login/i);
  //     act(() => {
  //       fireEvent.click(screen.getByText(/login/i));
  //     });
  //   });

  //   then('the user should see a descriptive error message', async () => {
  //     //expect(await screen.findByText(/bad username/i)).toBeInTheDocument()
  //   });
  // });
});
